@@ -157,6 +157,7 @@ class Distributor(GitRepo):
         force: bool = False,
         all: bool = False,
         yes: bool = False,
+        ignore_missing: bool = config.IGNORE_MISSING,
         dryrun: bool = False,
         versiononly: bool = False,
         verbose: bool = False,
@@ -178,6 +179,7 @@ class Distributor(GitRepo):
         :param force: If True, forces the distribution even if there are uncommitted changes.
         :param all: If True, processes all files, ignoring changes.
         :param yes: If True, automatically confirms prompts.
+        :param ignore_missing: If True, ignores missing source paths.
         :param dryrun: If True, simulates the distribution without making changes.
         :param versiononly: If True, only updates the version without changing the symlink.
         :param verbose: If True, provides detailed output.
@@ -211,8 +213,11 @@ class Distributor(GitRepo):
 
             source = entry.get(config.TAG_SOURCEPATH)
             dest = entry.get(config.TAG_DESTPATH)
-            if source is None or dest is None:
-                log.info(f"Target {name}: Missing source or dest path")
+            if source is None:
+                log.info(f"Target {name}: Missing source path")
+                continue
+            if dest is None:
+                log.info(f"Target {name}: Missing dest path")
                 continue
 
             # get target pipeline and options
@@ -243,7 +248,6 @@ class Distributor(GitRepo):
                     except Exception as e:
                         log.error(f"{e} resolving wildcard target {name}")
                         return False
-
             else:
                 try:
                     dest_resolved = util.sanitize_path(util.replace_vars(dest))
@@ -258,8 +262,14 @@ class Distributor(GitRepo):
                 )
 
                 if not os.path.exists(src_path):
-                    log.info(f"Target {name}: Source '{source}' does not exist")
-                    return False
+                    if target_options.get("ignore_missing", ignore_missing):
+                        log.info(
+                            f"Target {name}: Source '{source}' not found, skipping"
+                        )
+                        continue
+                    else:
+                        log.error(f"Target {name}: Source '{source}' does not exist")
+                        return False
 
                 if (
                     not show
@@ -310,6 +320,18 @@ class Distributor(GitRepo):
         for t in target_list:
             util.create_dest_folder(t.dest, dryrun, yes)
 
+            # determine if the commit hash will be used for matching versions
+            match_option = t.options.get("match", "commit")
+
+            # FIXME: fallback shim until match option is fully implemented
+            if match_option not in ["commit", "content"]:
+                raise ValueError(f"Unsupported version match option '{match_option}'")
+
+            if match_option == "commit":
+                commit_hash = self.short_head
+            else:
+                commit_hash = None  # don't use commit_hash in version matching
+
             if not dryrun and not show:
                 util.write_dist_info(
                     t.dest,
@@ -334,13 +356,13 @@ class Distributor(GitRepo):
             matches = util.find_matching_versions(
                 source_path=source_path,
                 dest=t.dest,
-                commit_hash=self.short_head,
+                commit_hash=commit_hash,
                 version_list=version_list,
                 force=force,
             )
 
             if matches and not force:
-                match_file, match_num, _ = matches[-1]
+                match_file, _, _ = matches[-1]
                 if os.path.islink(t.dest) and os.readlink(t.dest).endswith(
                     os.path.basename(match_file)
                 ):
