@@ -105,7 +105,7 @@ def copy_file(source: str, dest: str) -> None:
     try:
         destdir = os.path.dirname(dest)
         if not os.path.isdir(destdir):
-            os.makedirs(destdir)
+            os.makedirs(destdir, exist_ok=True)
         if os.path.islink(source):
             linkto = os.readlink(source)
             try:
@@ -132,10 +132,6 @@ def copy_file(source: str, dest: str) -> None:
 def copy_directory(source: str, dest: str, all_files: bool = False) -> None:
     """Recursively copies a directory (ignores hidden files).
 
-    Substitutes tokens in the file if substitute_tokens is True. Tokens are in
-    the form of {TOKEN} and can be replaced with environment variables or default
-    values.
-
     :param source: Path to source directory.
     :param dest: Path to destination directory.
     :param all_files: Copy all files, including hidden and ignorable files.
@@ -153,10 +149,6 @@ def copy_object(source: str, dest: str, all_files: bool = False) -> None:
     """Copies, or links, a file or directory recursively (ignores hidden
     files).
 
-    Substitutes tokens in the file if substitute_tokens is True. Tokens are in
-    the form of {TOKEN} and can be replaced with environment variables or default
-    values.
-
     :param source: Path to source file, link or directory.
     :param dest: Path to destination file or directory.
     :param all_files: Copy all files in a directory, including hidden and
@@ -718,6 +710,24 @@ def replace_vars(
     return "".join(result)
 
 
+def safe_copytree(src: str, dst: str):
+    """Safely copies a directory tree, avoiding infinite recursion, and skipping
+    ignorable file patterns defined in config.IGNORABLE.
+
+    :param src: Source directory path.
+    :param dst: Destination directory path.
+    """
+
+    if os.path.abspath(src) == os.path.abspath(dst):
+        raise ValueError(f"Source and destination are the same: {src}")
+
+    for file in walk(src, exclude_paths=[dst]):
+        relative_path = os.path.relpath(file, src)
+        target_path = os.path.join(dst, relative_path)
+        os.makedirs(os.path.dirname(target_path), exist_ok=True)
+        shutil.copy2(file, target_path)
+
+
 def yesNo(question: str) -> bool:
     """Displays question text to user and reads yes/no input.
 
@@ -746,7 +756,10 @@ def get_files(start: str, all_files: bool = False) -> List[str]:
 
 
 def walk(
-    path: str, exclude_ignorables: bool = True, followlinks: bool = False
+    path: str,
+    exclude_ignorables: bool = True,
+    followlinks: bool = False,
+    exclude_paths: Optional[List[str]] = None,
 ) -> Generator[str, None, None]:
     """Generator that yields relative file paths that are not ignorable.
     Will include nested directories and symbolic links to directories:
@@ -761,6 +774,7 @@ def walk(
     :param path: file system path.
     :param exclude_ignorables: exclude ignorable files.
     :param followlinks: follow symbolic links.
+    :param exclude_paths: list of paths to exclude from the search (optional).
     :return: generator of file paths.
     """
     if not is_ignorable(path) and os.path.isfile(path):
@@ -768,6 +782,8 @@ def walk(
     for dirname, dirs, files in os.walk(path, topdown=True, followlinks=followlinks):
         if exclude_ignorables and is_ignorable(dirname):
             continue
+        if exclude_paths and any(dirname.startswith(path) for path in exclude_paths):
+            continue
         for d in dirs:
             if exclude_ignorables and is_ignorable(d):
                 dirs.remove(d)
